;;;  File:	  ARROWS.LSP
;;;  Commands:  AR,ART
;;;  -----------------------------------------------------------
;;;  Date:	  1.8.93
;;;               27.12.94  Added control of PLINEWID variable
;;;               28.12.94  Added support for Model space arrows
;;;
;;;  Author:	  LRC
;;;  -----------------------------------------------------------
;;;  Use:	  Various programmes for drawing arrows eg.
;;;
;;;  AR:	  Arrowhead to point
;;;  ART:	  Text to arrowhead
;;;  -----------------------------------------------------------------------
;;;  Notes:     Requires the M_ENTSEL user defined function 
;;;  
;;;  
;;;  
;;;  
;;;  
;;;  
;;;  
;;;  ------------Reloads this lisp file------------------------
     (defun c:AA ()
     (load "arrows")
     )
;;;  ----------------------------------------------------------
;;;  

;;;  ---------------------------------------------------------------------------;
;;;  AR_ERR: Internal error handling.
;;;  ---------------------------------------------------------------------------;

(defun ar_err (msg)                     ; If an error (such as CTRL-C) occurs
                                        ; while this command is active...
  (if (/= msg "Function cancelled") 
    (princ (strcat "\nError: " msg))
  ) 
  (command "ucs" "")                  ; Restore World CS
     (setvar "CMDECHO" old_cmdecho)  ; Restore saved modes
     (setvar "BLIPMODE" old_blipmode)
     (setvar "ORTHOMODE" old_orthomode)
     (setvar "PLINEWID" old_plinewid)
  (setq *error* old_err)               ; Restore old *error* handler
  (princ)
) 

;;;  -----------------------------------------------------------
;;;  Support routine for drawing the arrow heads
;;;  -----------------------------------------------------------

(defun ARROW_HEAD ( / ang apt1 apt2 xpt1)
   (setq ang (angle pt1 pt2))
   (setq apt1 (polar pt1 (- ang (dtr 45)) (* 4 #ar_scale)))
   (setq apt2 (polar pt1 (+ ang (dtr 45)) (* 4 #ar_scale)))
   (command "pline" apt1 pt1 apt2 pt1 "")
   (setq head (entlast))
   (command "pedit" head "w" (* 0.35 #ar_scale) "")
)

;;;  -----------------------------------------------------------
;;;  Support routine for establishing endpoint adjacent to text
;;;  -----------------------------------------------------------

(defun TEXT_POINT (/ tb ll ur txtang dis cen txtend uc_txtend)
;  (setq textent (car (entsel "\nSelect text: "))) (terpri)
  (setq textent (car (m_entsel "\nSelect text: " "TEXT"))) (terpri)
  (command "ucs" "Entity" textent)
  (setq tb (textbox (list (cons -1 textent)))
        ll (car tb)
        ur (cadr tb)
    txtang (angle ll ur)
       dis (/ (distance ll ur) 2)
       cen (polar ll txtang dis)
  )
  (initget 1 "L R")
  (setq txtend (getkword "Which end of text?  Left or Right: "))
  (setq uc_txtend (strcase txtend))
  (if (/= uc_txtend "L")
      (setq txtpt (polar cen (dtr 0) (+ (* 1 #ar_scale) dis)))
      (setq txtpt (polar cen (dtr 180) (+ (* 1 #ar_scale) dis)))
  )

  (setq endpt (trans txtpt 1 0))
  (command "ucs" "p")
)


;;;  ---------------------------------------------------------
;;;  Main Program
;;;  AR:  MULTIPLE LINE ARROW - ARROWHEAD TO POINT
;;;  ---------------------------------------------------------

(defun C:AR (  / old_err old_cmdecho  old_blipmode old_orthomode 
                 pt1 pt2 endpt head ln1 ln2 tmode #ar_scale)

        (setq old_err *error*
              *error* ar_err)
        (setq old_cmdecho  (getvar "CMDECHO")
	      old_blipmode (getvar "BLIPMODE")
              old_orthomode (getvar "ORTHOMODE")
              old_plinewid (getvar "PLINEWID")
	)
	(setvar "CMDECHO" 0)
	(setvar "BLIPMODE" 0)
        (setvar "ORTHOMODE" 1)
        (setvar "PLINEWID" 0)

;; get Tilemode
;; if TM is O (ie. in Paper Space) set the #ar_scale variable to 1
;; if TM is 1 (ie. in Model Space) set the #ar_scale variable to dimscale
(setq tmode (getvar "TILEMODE"))
(if (and (= tmode 0) (= (getvar "CVPORT") 1))
    (setq #ar_scale 1)
    (setq #ar_scale (getvar "DIMSCALE"))
)

(setq pt1 (getpoint "\nArrow point: ")) (terpri)
(setq pt2 (getpoint pt1 "Next point: "))(terpri)
(command ".pline" pt1 pt2 "")
(setq ln1 (entlast))
(arrow_head)
(command ".pedit" ln1 "j" head "" "")
(setq ln1 (entlast))
(setq pt1 pt2)

(setq pt2 (getpoint pt1 "Next point: "))(terpri)

(while pt2
   (setvar "ORTHOMODE" 1)
   (command ".line" pt1 pt2 "")
   (setq ln2 (entlast))
   (command ".pedit" ln1 "j" ln2 "" "")
   (setq ln1 (entlast))
   (setq pt1 pt2)
   (setq pt2 (getpoint pt1 "Next point: "))(terpri)
)  ;while

      (setq *error* old_err)
      (setvar "CMDECHO" old_cmdecho)
      (setvar "BLIPMODE" old_blipmode)
      (setvar "ORTHOMODE" old_orthomode)
      (setvar "PLINEWID" old_plinewid)

    (prin1)
)



;;;  ---------------------------------------------------------
;;;  Main Program
;;;  ART:  MULTIPLE LINE ARROW - TEXT TO ARROWHEAD
;;;  ---------------------------------------------------------

(defun C:ART (/ old_err old_cmdecho  old_blipmode old_orthomode 
                pt1 pt2 pt3 endpt head ln1 ln2 tmode #ar_scale)

        (setq old_err *error*
              *error* ar_err)
        (setq old_cmdecho  (getvar "CMDECHO"))
	 (setq old_blipmode (getvar "BLIPMODE"))
        (setq old_orthomode (getvar "ORTHOMODE")) 
        (setq old_plinewid (getvar "PLINEWID"))

        (setvar "CMDECHO" 0)
        (setvar "BLIPMODE" 0)
        (setvar "ORTHOMODE" 1)
        (setvar "PLINEWID" 0)

;; get Tilemode
;; if TM is O (ie. in Paper Space) set the #ar_scale variable to 1
;; if TM is 1 (ie. in Model Space) set the #ar_scale variable to dimscale
(setq tmode (getvar "TILEMODE"))
(if (and (= tmode 0) (= (getvar "CVPORT") 1))
    (setq #ar_scale 1)
    (setq #ar_scale (getvar "DIMSCALE"))
)

(text_point)
(setq pt2 endpt)
(setq pt1 (getpoint pt2 "\nNext point: "))
(command ".pline" pt2 pt1 "")
(setq ln1 (entlast))
(setvar "ORTHOMODE" 1)
(setq pt3 (getpoint pt1 "\nNext point: "))

(while pt3
   (setvar "ORTHOMODE" 1)
   (command ".line" pt1 pt3 "")
   (setq ln2 (entlast))
   (command ".pedit" ln1 "j" ln2 "" "")
   (setq ln1 (entlast))
   (setq pt2 pt1)
   (setq pt1 pt3)
   (setq pt3 (getpoint pt1 "\nNext point: "))
)  ;while

   (arrow_head)
   (command ".pedit" ln1 "j" head "" "")

        (setq *error* old_err)
        (setvar "CMDECHO" old_cmdecho)
        (setvar "BLIPMODE" old_blipmode)
        (setvar "ORTHOMODE" old_orthomode)
        (setvar "PLINEWID" old_plinewid)

   (prin1)
)

;;;  ----------------------------------------------------------------------
;;;  END
;;;  ----------------------------------------------------------------------
